/*
	===========================
	nov 2008: adapted by Eric for walkie-talkie
	===========================
*/
#include <nds.h>
#include "smi_startup_viaipc.h"
#include "MessageQueue.h"
#include "wifi_hal.h"
#include "microphone.h"
#include "mytouch.h"

#include <common.h>

//---------------------------------------------------------------------------------
void startSound(int sampleRate, const void* data, u32 bytes, u8 channel, u8 vol,  u8 pan, u8 format) {
//---------------------------------------------------------------------------------
	SCHANNEL_TIMER(channel)  = TIMER7_S_COUNT(sampleRate);
	SCHANNEL_SOURCE(channel) = (u32)data;
	SCHANNEL_LENGTH(channel) = bytes >> 2 ;
	SCHANNEL_CR(channel)     = SCHANNEL_ENABLE | SOUND_REPEAT | SOUND_VOL(vol) | SOUND_PAN(pan) | SOUND_8BIT;
}


//---------------------------------------------------------------------------------
s32 getFreeSoundChannel() {
//---------------------------------------------------------------------------------
	int i;
	for (i=0; i<16; i++) {
		if ( (SCHANNEL_CR(i) & SCHANNEL_ENABLE) == 0 ) return i;
	}
	return -1;
}

int vcount;
touchPosition first,tempPos;

//---------------------------------------------------------------------------------
void VcountHandler() {
//---------------------------------------------------------------------------------
	static int lastbut = -1;
	uint16 but=0, x=0, y=0, xpx=0, ypx=0, z1=0, z2=0;
	
	Talkie->countC++;
	
	but = REG_KEYXY;

	if (!( (but ^ lastbut) & (1<<6))) {
 
		tempPos = myTouchReadXY();

		if ( tempPos.x == 0 || tempPos.y == 0 ) {
			but |= (1 <<6);
			lastbut = but;
		} else {
			x = tempPos.x;
			y = tempPos.y;
			xpx = tempPos.px;
			ypx = tempPos.py;
			z1 = tempPos.z1;
			z2 = tempPos.z2;
		}
		
	} else {
		lastbut = but;
		but |= (1 <<6);
	}

	IPC->touchX			= x;
	IPC->touchY			= y;
	IPC->touchXpx		= xpx;
	IPC->touchYpx		= ypx;
	IPC->touchZ1		= z1;
	IPC->touchZ2		= z2;
	IPC->buttons		= but;
}

//---------------------------------------------------------------------------------
void VblankHandler(void) {
//---------------------------------------------------------------------------------

	//static vuint32 heartbeat = 0;
	static vuint32 soundinit = 0;		
	u32 i;
	
	// Update the heartbeat
	//if (soundinit == 1) heartbeat++;
	
	//sound code  :)
	TransferSound *snd = IPC->soundData;
	
	
	u8 *micBuffer = Talkie->micBuffer;
	
	if (0 != snd) 
	{		
		for (i=0; i<snd->count; i++) 
		{
			s32 chan = getFreeSoundChannel();

			if (chan >= 0) 
			{
				startSound(snd->data[i].rate, snd->data[i].data, snd->data[i].len, chan, snd->data[i].vol, snd->data[i].pan, snd->data[i].format);
			}	
			
			Talkie->sndOffset = Talkie->micSampleCount; // offset of mic buffer pointer
			soundinit = 1;
		}
		
		IPC->soundData = 0;
	}

	if (0 != micBuffer)
	{		
		meStartRecording( micBuffer, SAMPLERATE);
		Talkie->micFrames = 0; // number of VBlanks since mic start 
		Talkie->micFrameOffset = 0; // number of samples since last VBlank
		Talkie->micBuffer = 0;
	}

}

//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------
	Talkie->micSampleCount = 0; // mic samples (audio index)
	IPC->mailData = 0;
	Talkie->micFrames = 0;      // incremented on VBlank
	Talkie->micFrameOffset = 0; // samples since last VBlank
	
	Talkie->sndOffset = 0;
	
	Talkie->countD = 0;

	IPC->countA = 0;
	// Reset the clock if needed
	rtcReset();

	//enable sound
	powerON(POWER_SOUND);
	SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
	IPC->soundData = 0;

	irqInit();
	SetYtrigger(80);
	vcount = 80;
	
	irqSet(IRQ_VBLANK, VblankHandler);
	//irqSet(IRQ_VCOUNT, VcountHandler);
	irqSet(IRQ_TIMER0, meProcessMicrophoneTimerIRQ);
	
	irqEnable(IRQ_VBLANK) ;
	//irqEnable(IRQ_VCOUNT);
	irqEnable(IRQ_TIMER0);
		
	
	while (IPC->toArm7buf == 0)
	{
		swiDelay(1);
	}
		
	IPC_Init() ;
	 
	LWIFI_Init() ;
	
	while (1) 
	{
		LWIFI_IPC_UpdateNOIRQ() ;
		swiWaitForVBlank();
		Talkie->micFrames++;
		Talkie->micFrameOffset = 0; // microphone samples since VBlank
		Talkie->countD++;
	} ;	
}


