/*---------------------------------------------------------------------------------
	$Id: microphone.c,v 1.5 2007/06/25 20:23:35 wntrmute Exp $

	Microphone control for the ARM7

Copyright (C) 2005
			Michael Noland (joat)
			Jason Rogers (dovoto)
			Dave Murphy (WinterMute)
			Chris Double (doublec)

This software is provided 'as-is', without any express or implied
warranty.  In no event will the authors be held liable for any
damages arising from the use of this software.

Permission is granted to anyone to use this software for any
purpose, including commercial applications, and to alter it and
redistribute it freely, subject to the following restrictions:

1. The origin of this software must not be misrepresented; you
	must not claim that you wrote the original software. If you use
	this software in a product, an acknowledgment in the product
	documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and
	must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source
	distribution.

	===========================
	nov 2008: adapted by Eric for walkie-talkie
	===========================

---------------------------------------------------------------------------------*/
#include <nds/timers.h>
#include "microphone.h"
#include <nds/ipc.h>
#include <nds.h>
#include <common.h>

extern void VcountHandler();

static int touchInterval = 136;

int framedelay = 0;
//---------------------------------------------------------------------------------
// Turn on the Microphone Amp. Code based on neimod's example.
//---------------------------------------------------------------------------------
void mePM_SetAmp(u8 control) {
//---------------------------------------------------------------------------------
	REG_IME = 0;

	SerialWaitBusy();
	
	REG_SPICNT = SPI_ENABLE | SPI_DEVICE_POWER | SPI_BAUD_1MHz | SPI_CONTINUOUS;
	REG_SPIDATA = PM_AMP_OFFSET;

	SerialWaitBusy();

	REG_SPICNT = SPI_ENABLE | SPI_DEVICE_POWER | SPI_BAUD_1MHz;
	REG_SPIDATA = control;

	SerialWaitBusy();
	
	REG_IME = 1;
}

void serWait()
{
	while (REG_SPICNT & SPI_BUSY)
			swiDelay(1);	
}
//---------------------------------------------------------------------------------
// Read a byte from the microphone. Code based on neimod's example.
//---------------------------------------------------------------------------------
u8 MIC_ReadData() {
//---------------------------------------------------------------------------------
	u16 result, result2;
	
	REG_IME = 0;
	
	serWait();
	
	REG_SPICNT = SPI_ENABLE | SPI_DEVICE_MICROPHONE | SPI_BAUD_2MHz | SPI_CONTINUOUS;
	REG_SPIDATA = 0xEC;  // Touchscreen command format for AUX

	serWait();

	REG_SPIDATA = 0x00;

	serWait();

	result = REG_SPIDATA;
	REG_SPICNT = SPI_ENABLE | SPI_DEVICE_TOUCH | SPI_BAUD_2MHz;
	REG_SPIDATA = 0x00;

	serWait();
	
	result2 = REG_SPIDATA;
	
	REG_IME = 1;

	return (((result & 0x7F) << 1) | ((result2>>7)&1));
}

extern volatile u8 *micBuffer;
volatile u8 *microphone_buffer = 0;
static u32 microphone_pos = 0;
static int microphone_buffer_length = 0;

//---------------------------------------------------------------------------------
void meStartRecording(u8* buffer, int length) {
//---------------------------------------------------------------------------------
	microphone_buffer_length = length;
	microphone_pos = 0;
	
	meMIC_On();
	
	// Set time to 0xFFFF - X, met X = period * 33513982
	TIMER0_DATA = TIMER7_N_COUNT(SAMPLERATE);
	TIMER0_CR = TIMER_ENABLE | TIMER_DIV_1 | TIMER_IRQ_REQ;  
	
	Talkie->micWriterPos = 0;
	
	microphone_buffer = buffer;
}

//---------------------------------------------------------------------------------
int meStopRecording() {
//---------------------------------------------------------------------------------
	TIMER0_CR &= ~TIMER_ENABLE;
	meMIC_Off();
	microphone_buffer = 0;
	return 0;
}

//---------------------------------------------------------------------------------
void  meProcessMicrophoneTimerIRQ() {
//---------------------------------------------------------------------------------
	if(microphone_buffer) 
	{
		// Read data from the microphone. Data from the Mic is unsigned, flipping
		// the highest bit makes it signed: microphone_buffer[microphone_pos] = MIC_ReadData() ^ 0x80;

		//microphone_buffer[microphone_pos] = 128;
		
		microphone_buffer[microphone_pos] = MIC_ReadData();
		if (++microphone_pos >= microphone_buffer_length) 
			microphone_pos = 0;
		
		Talkie->micSampleCount++; // audio counter
		IPC->mailData++;
		Talkie->micWriterPos = microphone_pos; // microphone buffer position
		Talkie->micFrameOffset++; // samples since VBlank
	}
	
	if (--touchInterval < 0)
	{
		touchInterval = 136;
		VcountHandler();
	}
	
}
