/*

File: drawboard.c
Version: v0.3 nov 2008

	Copyright (C) 2008
		Eric Giesselbach
		
	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any
	damages arising from the use of this software.

	Permission is granted to anyone to use this software for any
	purpose, including commercial applications, and to alter it and
	redistribute it freely, subject to the following restrictions:

	1.	The origin of this software must not be misrepresented; you
		must not claim that you wrote the original software. If you use
		this software in a product, an acknowledgment in the product
		documentation would be appreciated but is not required.
	2.	Altered source versions must be plainly marked as such, and
		must not be misrepresented as being the original software.
	3.	This notice may not be removed or altered from any source
		distribution.


*/

#include "nds.h"
#include <nds/arm9/console.h>
#include <stdio.h>

#include "MessageQueue.h"
#include "802.11.h"
#include "lobby.h"
#include <malloc.h>
#include <string.h>
#include <common.h>

#include "safe_malloc.h"
#include "buffers.h"
#include "mic.h"
#include "control.h"
#include "draw.h"
#include "drawboard.h"

//globals
extern u8* drawOutBuffer;            // draw data output buffer
extern u32 drawOutBufferLength;  // number of samples in wifi output buffer

// local

enum Action
{
	NONE,
 	SCREEN,
  	ERASER,
	PENCIL
};

		
tDrawData *drawData;

#define AREA (u16*)BG_BMP_RAM_SUB(0)

void draw_pixel(int x, int y, u8 activePen)
{
	uint16 blank = 0x7F7F;
	uint16 *ptr;
	uint16* buffer = AREA;
	int i,j;
		
	if (x < 2 || x > 229 || y > 191 + 35 || y < 40) return;
	
	buffer += y * 128 + (x >> 1);
	
	if (activePen != 0) 
	{
		activePen = 0xFF;	
		*buffer |= ( activePen << 8 * (x & 1) );
	}
	else
	{		
		// image has clipped margins on top, bottom...
		for (i = -1; i <= 1; i++)
		{
			for (j = -1; j <= 1; j++)
			{
				ptr = buffer + j * 128 + i;
				*ptr = blank;
			}
		}
	}	
}

void draw_bar(int x, int y, int length)
{
	int i, j;
	uint16* buffer = AREA;
	
	buffer += y * 256 + x;
	for(i = 0; i < 6; i++) // height
	{
		uint16* line = buffer + (128 * i);
		for(j = 0; j < length; j++) 
		{
			*line++ = RGB15(31, 31, 31) | BIT(15);
		}
		
		for(j = length; j < 75; j++) 
		{
			*line++ = RGB15(0, 0, 0) | BIT(15);
		}

	}
}

void clearscr()
{
	int i, j;
	uint16* buffer = AREA;
	
	uint16 value;
	
	for(i = 40; i < 192 + 35; i++) // height
	{
		uint16* line = buffer + (128 * i) + 1;
		
		value = 0x7F7F;
		
		for(j = 2; j < 116; j++) 
		{
			*line++ = value;
		}		
	}
}

void draw_line1(int x1, int y1, int x2, int y2, u8 activePen)
{
	int x, y;
	float dx, dy;
	int steps, i;
	
	dx = x2 - x1;
	dy = y2 - y1;
	
	steps = fmax(abs(x2 - x1), abs(y2 - y1));
	
	if (steps != 0)
	{
		dx = dx / steps;
		dy = dy / steps;
	}
	
	for( i = 0; i <= steps; i++) 
	{
		x = x1 + (int)((float)i * dx); 
		y = y1 + (int)((float)i * dy);
		 
		draw_pixel(x, y, activePen);	
	}
}

void draw_line(int x1, int y1, int x2, int y2, u8 activePen)
{	
	if (x1 > 229 && x2 > 229) // icon bar
		return;
	 
	drawData = (tDrawData*)&drawOutBuffer[drawOutBufferLength];
	
	drawData->x1 = x1;
	drawData->x2 = x2;
	drawData->y1 = y1;
	drawData->y2 = y2;
	drawData->pen = activePen;
	
	if ( drawOutBufferLength < AUDIOBUFLEN - sizeof(tDrawData) )
		drawOutBufferLength += sizeof(tDrawData);
	
	draw_line1(x1, y1, x2, y2, activePen);
}

enum Action selectAction(touchPosition touch)
{
	// 21 49 243
	if ( abs(touch.px - 243) < 7 && abs(touch.py - 164) < 10 )
			return SCREEN;
	else
		if ( screenState == DRAW && abs(touch.px - 243) < 7 && abs(touch.py - 21) < 10 )
			return ERASER;
	else
		if ( screenState == DRAW && abs(touch.px - 243) < 7 && abs(touch.py - 49) < 10 )
			return PENCIL;
	else		
		return NONE;
}

void switchScreen(enum Screen newState)
{
	if (newState == screenState) 
		return;
	
	screenState = newState;
	
	if (newState == DRAW)
	{
		dmaCopy(drawBitmap, BG_GFX_SUB, 256*256);
		dmaCopy(drawPal, BG_PALETTE_SUB, 256*2);
		consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
		BG_PALETTE_SUB[255] = RGB15(30,25,25);	//by default font will be rendered with color 255
		clearscr((u16*)BG_BMP_RAM_SUB(0));
	}
	else
	{
		dmaCopy(controlBitmap, BG_GFX_SUB, 256*256);
		dmaCopy(controlPal, BG_PALETTE_SUB, 256*2);
		consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);
		BG_PALETTE_SUB[255] = RGB15(30,25,25);	//by default font will be rendered with color 255
	}
	
}

void receiveText(unsigned char *data, int length, LPLOBBY_USER from)
{
	int pos = 0;
	tDrawData *drawData;
	
	if (screenState == CONTROL) 
		switchScreen(DRAW);
	
	while ( pos < length )
	{
		drawData = (tDrawData*)(data + pos);
		draw_line1(drawData->x1, drawData->y1, drawData->x2, drawData->y2, drawData->pen);
		pos += sizeof(tDrawData);
	}	
}
	

touchPosition touch;
int prevpx = 0, prevpy = 0;
int touchDelay = 0;
u8 activePen = 1;
	
		
//------------------------------------------------------
void handleTouch()
{		
		touch=touchReadXY();	
		if (touchDelay > 0) touchDelay--;
				
		if (screenState == DRAW && (touch.px != 0 || touch.py != 0) )
		{		
			if (prevpx != 0)
				draw_line(touch.px, 38 + touch.py, prevpx, 38 + prevpy, activePen);
			else
				draw_line(touch.px, 38 + touch.py, touch.px, 38 + touch.py, activePen);			
		}
		
		prevpx = touch.px;
		prevpy = touch.py;
				
		
		IPC->countB = touch.px;
		
		if ( (keysHeld() & KEY_TOUCH) == 0)
		{
			prevpx = 0;
			prevpy = 0;	
		}	

		enum Action act = selectAction(touch);
		
		if ( touchDelay == 0 && act != NONE )
		{
			touchDelay = 30;
			
			switch(act)
			{
				case SCREEN:
					switchScreen( screenState==CONTROL?DRAW:CONTROL );
				break;
				case ERASER:
					activePen = 0;
				break;
				case PENCIL:
					activePen = 1;
				break;
				case NONE:
				break;
			}
		}
}			
